from dataclasses import dataclass
from typing import Callable

def default_error_presenter(error: Exception) -> bool:
    return False # let caller re-raise error

@dataclass
class CurliConfig:
    error_presenter: Callable[[Exception], bool] = default_error_presenter



import httpx

from ._base import ApiBeanError


class TransportError(ApiBeanError):
    pass


class ConnectionError(TransportError):
    code = "connection_failed"
    message_template = "Cannot connect to {url}"
    hint = "Check server address or make sure the server is running"


class ReqTimeoutError(TransportError):
    code = "read_timeout"
    message_template = "Request to {url} timed out after {timeout}(s)"
    hint = "Try increasing timeout or check server performance"


def map_httpx_exception(exc: httpx.RequestError, timeout = None, **kwargs) -> TransportError:
    request = exc.request
    url = str(request.url) if request else "<unknown>"

    if isinstance(exc, httpx.ConnectError):
        return ConnectionError(
            message_data={"url": url},
            cause=exc,
        )

    if isinstance(exc, httpx.ReadTimeout):
        return ReqTimeoutError(
            message_data={"url": url, "timeout": timeout or 5.0},
            cause=exc,
        )

    return TransportError(
        message=str(exc),
        cause=exc,
    )



import httpx

from .engine import Agent, Curli, Store, Tools

class Space:
    account = Store(profile = "anon")
    session = Store(profile = "main")

space = Space()
curli = Curli(httpx, session_store=space.session, account_store=space.account)
agent = Agent(curli)
tools = Tools()

curli.default(headers={
    "accept": "application/json",
    "Content-Type": "application/json",
})



from ._decorators import deprecated

class Store():
    def __init__(self, profile="main"):
        self._default = dict()
        self._storage = dict()
        self._profile = profile
        if self._profile:
            self._storage[self._profile] = dict()

    @property
    def profile(self):
        return self._profile

    @profile.setter
    def profile(self, value):
        self._profile = value

    @property
    def profiles(self):
        return list(self._storage.keys())

    @deprecated
    def globals(self, **kwargs):
        self.default(**kwargs)

    def default(self, **kwargs):
        self._default.update(**kwargs)

    def reset(self):
        self._storage = dict()
        if self._profile:
            self._storage[self._profile] = dict()

    def _get_storage_of_profile(self):
        if self._profile not in self._storage:
            self._storage[self._profile] = dict()
        return self._storage[self._profile]

    def update(self, *args, **kwargs):
        self._get_storage_of_profile().update(*args, **kwargs)

    def __contains__(self, item):
        return item in self._get_storage_of_profile()

    def __getitem__(self, key):
        return self._get_storage_of_profile().get(key, self._default.get(key, None))

    def __setitem__(self, key, value):
        _storage = self._get_storage_of_profile()
        _storage[key] = value

    def __delitem__(self, key):
        _storage = self._get_storage_of_profile()
        if key in _storage:
            del _storage[key]

    def __getattr__(self, name):
        """Intercepts attribute access and forwards it to the storage of current profile."""
        return getattr(self._get_storage_of_profile(), name)



import json
import httpx

from ._helpers import Curlify

class Tools:

    def print_json(self, data):
        print(json.dumps(data, indent=2))

    def print_body(self, response):
        self.print_json(response.json())

    def print_response(self, response):
        if response.status_code == httpx.codes.OK:
            print(json.dumps(response.json(), indent=2))
        else:
            print(f"Error: { json.dumps(response.json(), indent=2) }")

    def print_curl(self, req_or_resp: httpx.Request|httpx.Response):
        request = req_or_resp
        if isinstance(req_or_resp, httpx.Response):
            request = req_or_resp.request
        print(Curlify(request).to_curl())

    def print_session_item(self, store, item_name):
        item = store[item_name]
        if isinstance(item, dict):
            print(f'store.session["{ item_name }"]:')
            self.print_json(item)
        else:
            print(f'store.session["{ item_name }"]: { item }')



from typing import List, Optional
from functools import reduce

import json
import re

import httpx

from ._consts import HK_REQUEST_ID
from ._utils import comma_delimited_string

class RequestWrapper:
    def __init__(self, wrapped_object, session_store, account_store, **kwargs):
        self._wrapped_object = wrapped_object
        self._session_store = session_store
        self._account_store = account_store
        self._other_kwargs = kwargs or dict()

    def __getattr__(self, name):
        """Intercepts attribute access and forwards it to the wrapped object."""
        return getattr(self._wrapped_object, name)

    def __setattr__(self, name, value):
        """Intercepts attribute assignment and forwards it to the wrapped object."""
        if name == "_wrapped_object":
            # Allow assignment of the wrapped object itself.
            super().__setattr__(name, value)
        else:
            # Forward other assignments to the wrapped object.
            setattr(self._wrapped_object, name, value)

    def __repr__(self):
        """Returns the representation of the wrapped object."""
        return repr(self._wrapped_object)

    def __str__(self):
        """Returns the string representation of the wrapped object."""
        return str(self._wrapped_object)

    @property
    def __class__(self):
        """Preserves the class of the wrapped object."""
        return self._wrapped_object.__class__

    def send(self):
        return ResponseWrapper(httpx.Client().send(self._wrapped_object),
                session_store=self._session_store,
                account_store=self._account_store)

    def to_curl(self):
        """to_curl function returns a string of curl to execute in shell."""
        return Curlify(self._wrapped_object, **self._other_kwargs).to_curl()

    def print_curl(self):
        print(self.to_curl())


class ResponseWrapper:
    def __init__(self, wrapped_object, session_store, account_store, **kwargs):
        self._wrapped_object = wrapped_object
        self._session_store = session_store
        self._account_store = account_store
        self._other_kwargs = kwargs or dict()

    def __getattr__(self, name):
        """Intercepts attribute access and forwards it to the wrapped object."""
        return getattr(self._wrapped_object, name)

    def __setattr__(self, name, value):
        """Intercepts attribute assignment and forwards it to the wrapped object."""
        if name == "_wrapped_object":
            # Allow assignment of the wrapped object itself.
            super().__setattr__(name, value)
        else:
            # Forward other assignments to the wrapped object.
            setattr(self._wrapped_object, name, value)

    def __repr__(self):
        """Returns the representation of the wrapped object."""
        return repr(self._wrapped_object)

    def __str__(self):
        """Returns the string representation of the wrapped object."""
        return str(self._wrapped_object)

    @property
    def __class__(self):
        """Preserves the class of the wrapped object."""
        return self._wrapped_object.__class__

    def _assert_response(self):
        if self._wrapped_object is None:
            raise RuntimeError(
                "The original response instance has not been set on this response wrapper."
            )
        return self._wrapped_object

    def _assert_response_body(self):
        body = self._assert_response().json()
        return body

    def get_id(self):
        return self.get_value_of("id")

    def get_value_of(self, field_name):
        body = self._assert_response_body()
        if field_name not in body:
            raise RuntimeError(
                f"The returned data does not contain the [{ field_name }] field."
            )
        return body.get(field_name)

    def capture_id_refs(self, name_of_id_refs:Optional[str] = None, name_of_key_field = "email", reset: bool = False):
        if not self._wrapped_object.is_success:
            self.print()
            raise RuntimeError("The response is not sussess")
        if name_of_id_refs is None:
            name_of_id_refs = self._build_name_of_id_refs()
        if name_of_id_refs not in self._session_store or reset:
            self._session_store[name_of_id_refs] = dict()
        self._session_store[name_of_id_refs].update(**self._extract_ids_map(name_of_key_field))
        return self

    def _build_name_of_id_refs(self):
        return f"{ self._extract_model_name(self._wrapped_object) }_ids_of"

    def _extract_model_name(self, r):
        urlpath = r.request.url.path
        return urlpath.split('/')[-1]

    def _extract_ids_map(self, name_of_key_field="email", name_of_key_fields: Optional[List] = None):
        if not name_of_key_fields:
            name_of_key_fields = comma_delimited_string(name_of_key_field)
        return reduce(lambda acc, item: {**acc, ",".join(item[k] for k in name_of_key_fields): item["id"]},
                self._get_list_of_items(),
                dict())

    def _get_list_of_items(self):
        resp_body = self._wrapped_object.json()
        if "founds" in resp_body:
            items = resp_body["founds"]
        else:
            items = [ resp_body ]
        return items

    def get_request_id(self, from_response=False):
        if from_response:
            headers = self._wrapped_object.headers
        else:
            headers = self._wrapped_object.request.headers
        return headers.get(HK_REQUEST_ID.lower(), None)

    def get_request_id_pair(self):
        return self.get_request_id(), self.get_request_id(from_response=True)

    def print(self, details: bool = False):
        response = self._wrapped_object
        if response.is_success:
            print(response)
        else:
            print(response)
            print(json.dumps(response.json(), indent=2))

    def print_body(self):
        print(json.dumps(self._wrapped_object.json(), indent=2))

    def print_curl(self):
        print(Curlify(self._wrapped_object.request, **self._other_kwargs).to_curl())


class ErrorWrapper:
    def __init__(self, error, *args, **kwargs):
        self._error = error

    def print(self, details: bool = False):
        pass

    def print_body(self):
        pass

    def print_curl(self):
        pass


class Curlify:
    DEFAULT_EXCLUDE_HEADERS = (
        "host",
        "content-length",
        "accept-encoding",
        "connection",
        "user-agent",
    )

    def __init__(self, request, exclude_headers = DEFAULT_EXCLUDE_HEADERS,
            compressed=False, verified=True, timeout=None, **kwargs):
        self.req = request
        self.exclude_headers = exclude_headers
        self.compressed = compressed
        self.verified = verified
        self.timeout = timeout

    def to_curl(self) -> str:
        """to_curl function returns a string of curl to execute in shell.
        """
        return self.build()

    def build(self) -> str:
        """build curl command string

        Returns:
            str: string represents curl command
        """
        quote = f'curl -X {self.req.method} "{self.req.url}" -H {self.headers()}'
        if self.req.method != 'GET':
            quote = quote + f" -d '{self.decode_body()}'"

        parts = re.split(r'\s(?=-[dH])', quote)

        if self.timeout:
            parts.append(f"--max-time {self.timeout}")

        if self.compressed:
            parts.append("--compressed")
        if not self.verified:
            parts.append("--insecure")

        quote = " \\\n".join(parts)
        return quote

    def headers(self) -> str:
        """convert the request's headers to string

        Returns:
            str: return string of headers
        """
        headers = [f'"{k}: {v}"' for k, v in self.req.headers.items() if k not in self.exclude_headers]

        return " -H ".join(headers)

    def decode_body(self):
        body = self.read_body()

        if body and isinstance(body, bytes):
            content_type = self.req.headers.get("content-type", None)
            if content_type and content_type == "application/json":
                return json.dumps(json.loads(body.decode()), indent=2)
            return body.decode()

        return body

    def read_body(self):
        if hasattr(self.req, "body"):
            return self.req.body

        return self.req.read()



from datetime import datetime, timezone


def to_datetime(datetime_str: str|None) -> datetime|None:
    if str is None:
        return None
    return datetime.fromisoformat(datetime_str).replace(tzinfo=timezone.utc)


def get_now() -> datetime:
    return datetime.now(timezone.utc)


def normalize_header(headers: dict[str, str], canonical_key: str) -> dict[str, str]:
    normalized = {}
    for key, value in headers.items():
        if key.casefold() == canonical_key.casefold():
            normalized[canonical_key] = value
            break
        else:
            normalized[key] = value
    return normalized


def normalize_headers(headers: dict[str, str], canonical_keys: list[str]) -> dict[str, str]:
    normalized = {}
    for key, value in headers.items():
        for canonical in canonical_keys:
            if key.casefold() == canonical.casefold():
                normalized[canonical] = value
                break
        else:
            # Không khớp: giữ nguyên key gốc
            normalized[key] = value
    return normalized


def comma_delimited_string(s: str):
    return comma_delimited_string_to_cls_list(s, str)


def comma_delimited_string_to_cls_list(s: str, as_cls = str):
    return [as_cls(x.strip()) for x in s.split(',')] if s is not None else None


def comma_delimited_string_to_int_list(s: str):
    return comma_delimited_string_to_cls_list(s, int)



from typing import Literal

Severity = Literal["error", "warning", "info"]

class ApiBeanError(Exception):
    """
    Base class for all apibean-client errors.
    """

    # --- Required fields ---
    code: str = "unknown_error"
    message: str = "An error occurred"
    severity: Severity = "error"

    # --- Optional fields ---
    title: str | None = None
    detail: str | None = None
    hint: str | None = None

    message_template: str | None = None

    def __init__(self, *,
        message=None,
        message_template=None,
        message_data=None,
        cause: Exception | None = None,
        detail: str | None = None,
        hint: str | None = None
    ):
        self.message_data = message_data or {}

        message_template = message_template or self.message_template

        if message:
            self.message = message
        elif message_template:
            self.message = message_template.format(**self.message_data)
        else:
            self.message = self.__class__.__name__

        self.cause = cause

        if detail:
            self.detail = detail
        if hint:
            self.hint = hint

        super().__init__(self.message)



from typing import Self

from ._consts import JF_USERNAME
from ._consts import JF_PASSWORD
from ._consts import JF_ACTIVATION_CODE
from ._consts import JF_ID
from ._consts import JF_EMAIL
from ._consts import JF_ACCESS_TOKEN
from ._consts import JF_REFRESH_TOKEN
from ._consts import JF_EXPIRATION

from ._curli import Curli
from ._decorators import deprecated
from ._utils import to_datetime, get_now

class Agent:
    """The Agent object class wraps some of the operations of authenticating users and 
    switching between these accounts during interactions.

    Properties:
    - `ACCOUNT_AUTH_FIELDS`: The names of the fields containing user authentication
        information of an account.
        In which:

        * id: Unique identifier of the user in the system (usually a UUID code).
        * email: Unique email address of the user, this is also the login name.
        * access_token: Access code according to the JWT protocol.
        * refresh_token: Secret code used to refresh the access_token.
        * expiration: Time when the access_token code expires. The access_token must
            be refreshed before this time.
    """

    ACCOUNT_AUTH_FIELDS = [JF_ID, JF_EMAIL, JF_ACCESS_TOKEN, JF_REFRESH_TOKEN, JF_EXPIRATION]

    def __init__(self, curli: Curli):
        self._curli = curli

    def as_account(self, *args, **kwargs) -> Self:
        self._curli.as_account(*args, **kwargs)
        return self

    def in_session(self, *args, **kwargs) -> Self:
        self._curli.in_session(*args, **kwargs)
        return self

    @property
    def _account(self):
        return getattr(self._curli, "_account")

    @property
    def _session(self):
        return getattr(self._curli, "_session")

    def is_authenticated(self) -> bool:
        for required_field in [JF_ACCESS_TOKEN, JF_REFRESH_TOKEN, JF_EXPIRATION]:
            if required_field not in self._account:
                return False
        return True

    def is_still_valid(self) -> bool | None:
        if not self.is_authenticated():
            return None
        expires = to_datetime(self._account.get(JF_EXPIRATION))
        if expires <= get_now():
            return False
        return True

    @deprecated
    def auth(self, *args, **kwargs):
        return self.login(*args, **kwargs)

    def login(self, username = None, password = None, json = None, url:str = "auth/login", **kwargs):
        body = json if isinstance(json, dict) else dict()

        if username:
            body[JF_USERNAME] = username
        if password:
            body[JF_PASSWORD] = password

        kwargs = dict(filter(lambda item: item[0] != 'json', kwargs.items()))
        kwargs.update(access_token=None)

        r = self._curli.post(url, json = body, **kwargs)

        if r.is_success:
            self.as_account(**self._extract_cached(r))

        return r

    def _extract_cached(self, r):
        body = r.json()
        params = {key: body[key] for key in self.ACCOUNT_AUTH_FIELDS if key in body}
        for field_name in [JF_ID, JF_ACCESS_TOKEN]:
            self._check_available(params, field_name)
        return params

    def _check_available(self, data, field_name):
        if field_name not in data:
            raise RuntimeError(f"{ field_name } not found")
        field_value = data.get(field_name)
        if field_value is None:
            raise RuntimeError(f"{ field_name } is null")
        if not field_value:
            raise RuntimeError(f"{ field_name } is empty")

    def change_password(self, current_password, new_password, url:str = "auth/change-password", **kwargs):
        return self._curli.post(url,
                json=dict(current_password=current_password, new_password=new_password),
                **kwargs)

    def refresh_token(self, url:str = "auth/refresh-token", **kwargs):
        email = self._account.get(JF_EMAIL)
        if not email:
            raise RuntimeError(f"{ JF_EMAIL } not found")

        refresh_token = self._account.get(JF_REFRESH_TOKEN)
        if not refresh_token:
            raise RuntimeError(f"{ JF_REFRESH_TOKEN } not found")

        r = self._curli.post(url, json=dict(email=email, refresh_token=refresh_token), **kwargs)

        if r.is_success:
            self.as_account(**self._extract_cached(r))

        return r

    def logout(self, url:str = "auth/logout", **kwargs):
        r = self._curli.get(url, **kwargs)
        if r.is_success:
            self.as_account(access_token = None)
        return r

    def activate_user_id(self, user_id, password = None, url:str = "auth/activate", **kwargs):
        """Any user excepts anon & root could use this API to activate his account
        """
        backup_profile = self._account.profile
        user_response = self._curli.as_account("root").get("user/" + user_id)
        self._curli.as_account(backup_profile)
        activation_code = user_response.json().get(JF_ACTIVATION_CODE)
        return self.activate(activation_code, password, url=url, **kwargs)

    def activate(self, activation_code, password = None, url:str = "auth/activate", **kwargs):
        r = self._curli.post(url, json = dict(activation_code=activation_code, password=password),
                access_token=None, **kwargs)
        return r




JF_BASE_URL = 'base_url'
JF_ID = 'id'
JF_EMAIL = 'email'
JF_USERNAME = 'username'
JF_PASSWORD = 'password'
JF_ACTIVATION_CODE = 'activation_code'
JF_ACCESS_TOKEN = 'access_token'
JF_REFRESH_TOKEN = 'refresh_token'
JF_EXPIRATION = 'expiration'

HK_AUTHORIZATION = 'Authorization'
HK_REQUEST_ID = 'X-Request-Id'.lower()



import warnings
from functools import wraps

def deprecated(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        warnings.warn(
            f"{func.__name__} is deprecated and will be removed in future versions",
            category=DeprecationWarning,
            stacklevel=2
        )
        return func(*args, **kwargs)
    return wrapper



from typing import Self
from uuid import uuid4

import urllib.parse
import httpx

from ._consts import JF_BASE_URL
from ._consts import JF_ACCESS_TOKEN
from ._consts import HK_AUTHORIZATION
from ._consts import HK_REQUEST_ID
from ._config import CurliConfig
from ._decorators import deprecated
from ._helpers import RequestWrapper
from ._helpers import ResponseWrapper
from ._helpers import ErrorWrapper
from ._store import Store
from ._utils import normalize_header

from ..errors import map_httpx_exception


class Curli:

    def __init__(self, invoker, session_store: Store, account_store: Store, config: CurliConfig|None = None):
        self._invoker = invoker
        self._session = session_store
        self._account = account_store
        self._config = config or CurliConfig()

    @property
    def config(self):
        return self._config

    @property
    def invoker(self):
        return self._invoker

    @invoker.setter
    def invoker(self, value):
        self._invoker = value

    @deprecated
    def globals(self, **kwargs) -> Self:
        return self.default(**kwargs)

    def default(self, **kwargs) -> Self:
        self._session.default(**kwargs)
        return self

    def as_account(self, profile = None, **kwargs) -> Self:
        if profile is not None:
            self._account.profile = profile
        self._account.update(**kwargs)
        return self

    def in_session(self, profile = None, **kwargs) -> Self:
        if profile is not None:
            self._session.profile = profile

        kwargs = dict(kwargs)
        if JF_BASE_URL in kwargs:
            if kwargs[JF_BASE_URL] is not None:
                self._session[JF_BASE_URL] = kwargs[JF_BASE_URL]
            del kwargs[JF_BASE_URL]
        self._session.update(**kwargs)

        return self

    def _build_params(self, url, *args, headers = None, **kwargs):
        base_url = kwargs.get(JF_BASE_URL, self._session[JF_BASE_URL])
        if not url.startswith('http') and base_url:
            url = urllib.parse.urljoin(base_url + '/', url.lstrip('/'))

        if not isinstance(headers, dict):
            headers = {}

        # normalize request headers and apply default header values
        headers = normalize_header(headers, HK_REQUEST_ID)

        session_headers = self._session['headers']
        if isinstance(session_headers, dict):
            headers = {**session_headers, **headers}

        access_token = kwargs.get(JF_ACCESS_TOKEN, self._account[JF_ACCESS_TOKEN])
        if access_token and isinstance(access_token, str):
            headers = {HK_AUTHORIZATION: f"Bearer {access_token}", **headers}
        if JF_ACCESS_TOKEN in kwargs:
            del kwargs[JF_ACCESS_TOKEN]

        headers = {HK_REQUEST_ID: str(uuid4()), **headers}

        other_kwargs = { key: value for key, value in kwargs.items() if key in ("timeout")}

        return (url, args, dict(kwargs, headers=headers), other_kwargs)

    def _wrap_response(self, response, **others):
        return ResponseWrapper(response, session_store=self._session, account_store=self._account, **others)

    def _wrap_request(self, request, **others):
        return RequestWrapper(request, session_store=self._session, account_store=self._account, **others)

    def _wrap_error(self, error, **others):
        return ErrorWrapper(error)

    #--------------------------------------------------------------------------

    class PrepareObject:
        def __init__(self, parent):
            self._parent = parent

        def request(self, method, url, *args, **kwargs):
            return self._parent.pre_request(method=method, url=url, *args, **kwargs)

        def get(self, url, *args, **kwargs):
            return self.request("GET", url, *args, **kwargs)

        def head(self, url, *args, **kwargs):
            return self.request("HEAD", url, *args, **kwargs)

        def options(self, url, *args, **kwargs):
            return self.request("OPTIONS", url, *args, **kwargs)

        def post(self, url, *args, **kwargs):
            return self.request("POST", url, *args, **kwargs)

        def put(self, url, *args, **kwargs):
            return self.request("PUT", url, *args, **kwargs)

        def patch(self, url, *args, **kwargs):
            return self.request("PATCH", url, *args, **kwargs)

        def delete(self, url, *args, **kwargs):
            return self.request("DELETE", url, *args, **kwargs)

    @property
    def prepare(self):
        return self.PrepareObject(self)

    def pre_request(self, method, url, *args, **kwargs):
        url, args, kwargs, others = self._build_params(url, *args, **kwargs)
        return self._wrap_request(httpx.Request(method=method, url=url, *args, **kwargs), **others)

    def pre_get(self, url, *args, **kwargs):
        return self.pre_request("GET", url, *args, **kwargs)

    def pre_head(self, url, *args, **kwargs):
        return self.pre_request("HEAD", url, *args, **kwargs)

    def pre_options(self, url, *args, **kwargs):
        return self.pre_request("OPTIONS", url, *args, **kwargs)

    def pre_post(self, url, *args, **kwargs):
        return self.pre_request("POST", url, *args, **kwargs)

    def pre_put(self, url, *args, **kwargs):
        return self.pre_request("PUT", url, *args, **kwargs)

    def pre_patch(self, url, *args, **kwargs):
        return self.pre_request("PATCH", url, *args, **kwargs)

    def pre_delete(self, url, *args, **kwargs):
        return self.pre_request("DELETE", url, *args, **kwargs)

    #--------------------------------------------------------------------------

    def request(self, method, url, *args, **kwargs):
        url, args, kwargs, others = self._build_params(url, *args, **kwargs)
        try:
            return self._wrap_response(self._invoker.request(method, url, *args, **kwargs), **others)
        except httpx.RequestError as exc:
            error = map_httpx_exception(exc, **others)
            if callable(self._config.error_presenter):
                if self._config.error_presenter(error):
                    return self._wrap_error(error, **others)
            raise error from exc

    def get(self, url, *args, **kwargs):
        return self.request("GET", url, *args, **kwargs)

    def head(self, url, *args, **kwargs):
        return self.request("HEAD", url, *args, **kwargs)

    def options(self, url, *args, **kwargs):
        return self.request("OPTIONS", url, *args, **kwargs)

    def post(self, url, *args, **kwargs):
        return self.request("POST", url, *args, **kwargs)

    def put(self, url, *args, **kwargs):
        return self.request("PUT", url, *args, **kwargs)

    def patch(self, url, *args, **kwargs):
        return self.request("PATCH", url, *args, **kwargs)

    def delete(self, url, *args, **kwargs):
        return self.request("DELETE", url, *args, **kwargs)



from ._base import ApiBeanError
from ._base import Severity

from ._transport import TransportError
from ._transport import ConnectionError
from ._transport import ReqTimeoutError
from ._transport import map_httpx_exception
